/*
Bmp_dehide and Bmp_hide are sample programs for hiding data in bmp-images
Copyright (c) 2006 Konstantin Plotnikov aka Kost

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License only.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

You can contact me by e-mail: kost]at[hackerdom]dot[ru
or paper mail: 620083, Russia, g. Ekaterinburg, K-83, pr. Lenina, 51
*/

#include "stdafx.h"
#include <fstream>
#include "iostream.h"
#include <string.h>
#include <stdlib.h>

void ar_cpy(unsigned char *ar1, unsigned char *ar2, int length){
	for (int i=0; i < length; i++)
		ar1[i]=ar2[i];
}

int read_inversed(unsigned char * arr, int amount){
	int result=0;;
	for (int i = 0; i < amount; i++){
		result += (arr[i] << (i*8));
	}
	return result;
}

unsigned char read_bit(unsigned char * arr, unsigned int numb){
	unsigned int byte = numb/8;
	unsigned int bit = numb%8;
	return (arr[byte] >> bit ) & 1;
}

int main(int argc, char* argv[])
{
	char picture[256];
	char file[256];
	
	if ( argc > 3){
		cout << "Usage: bmp_dehide.exe <picture.bmp <result>>\n";
		return 1;
	}

	if (argc >= 2){
		strcpy(picture, argv[1]);
		if (argc == 3)
			strcpy(file, argv[2]);
		else strcpy(file, "hidden_inf");
	}
	
	else{
		strcpy(picture, "hidden.bmp");
		strcpy(file, "hidden_inf");
	}

	unsigned char buf[1100];
	int data_offset, image_size;
	FILE * pict, * hidden_file;
	if ( ( pict = fopen ( picture, "rb" ) ) == NULL ) {
		cout << "Error in reading picture\n";
		return 1;
	}

	if ( ( hidden_file = fopen ( file, "w+b" ) ) == NULL ) {
		cout << "Error in opening file\n";
		return 1;
	}

	fread ( buf, sizeof ( char ), 14, pict );
	if ( buf[0] != 'B' || buf[1] != 'M'){
		cout << "Error. Not an expected BMP file is selected";
		return 1;
	}

	image_size = (buf[3] << 8) + buf[2];
	unsigned char * out = (unsigned char *)malloc(image_size);
	
	data_offset = read_inversed(buf+0xA,4);
	fread(buf+14, sizeof(char), data_offset-14, pict);
	
	unsigned int image_width = read_inversed(buf+0x12,4);
	unsigned int image_height = read_inversed(buf+0x16,4);

	int bits_per_pixel = read_inversed(buf+0x1C, 2);
	if (bits_per_pixel != 24){
		cout << "Only 24-bit BMP pictures are expected";
		return 1;
	}

	int compression = read_inversed(buf+0x1E,4);
	if (compression != 0){
		cout << "Only not compressed BMP pictures are expected";
		return 1;
	}

	ar_cpy(out, buf, data_offset);
	
	unsigned int hidden_data_size = read_inversed(buf+6,4);

	unsigned char * hidden_data = (unsigned char *)malloc(hidden_data_size+1);
	for (int i=0, j=-1; i < hidden_data_size; i++)
		hidden_data[i] = 0;
	
	for (i=0; i < hidden_data_size*8; i++){
		if (i%8==0) j++;
		fread(buf, sizeof(char), 1, pict);
		unsigned char bit = read_bit(buf,0);
		hidden_data[j] += bit << i%8;
	}
		
	for(i=0; i < hidden_data_size; i++)
		fwrite(hidden_data+i, sizeof( char ), 1, hidden_file);
	hidden_data[j+1]=0;
	free(out);
	free(hidden_data);
	fclose(pict);
	fclose(hidden_file);
	return 0;
}
